import click
from clapper.click import ResourceOption, verbosity_option
from clapper.logging import setup
from mednet.libs.common.scripts.click import ConfigCommand
from mednet.libs.common.scripts.train import reusable_options

logger = setup("mednet", format="%(levelname)s: %(message)s")


@click.command(
    entry_point_group="mednet.libs.segmentation.config",
    cls=ConfigCommand,
    epilog="""Examples:

1. Train a lwnet model with the frive dataset, on a GPU (``cuda:0``):

   .. code:: sh

      mednet segmentation train -vv lwnet drive --batch-size=4 --device="cuda:0"
""",
)
@reusable_options
@verbosity_option(logger=logger, cls=ResourceOption, expose_value=False)
def train(
    model,
    output_folder,
    epochs,
    batch_size,
    accumulate_grad_batches,
    drop_incomplete_batch,
    datamodule,
    validation_period,
    device,
    cache_samples,
    seed,
    parallel,
    monitoring_interval,
    augmentations,
    **_,
) -> None:  # numpydoc ignore=PR01
    """Train an CNN to perform image segmentation.

    Training is performed for a configurable number of epochs, and
    generates checkpoints.  Checkpoints are model files with a .ckpt
    extension that are used in subsequent tasks or from which training
    can be resumed.
    """
    from lightning.pytorch import seed_everything
    from mednet.libs.common.engine.device import DeviceManager
    from mednet.libs.common.engine.trainer import run
    from mednet.libs.common.scripts.train import (
        get_checkpoint_file,
        load_checkpoint,
        setup_datamodule,
    )
    from mednet.libs.common.scripts.utils import save_json_metadata

    seed_everything(seed)

    # report model/transforms options - set data augmentations
    logger.info(f"Network model: {type(model).__module__}.{type(model).__name__}")
    model.augmentation_transforms = augmentations

    device_manager = DeviceManager(device)

    # reset datamodule with user configurable options
    setup_datamodule(
        datamodule,
        model,
        batch_size,
        drop_incomplete_batch,
        cache_samples,
        parallel,
    )

    checkpoint_file = get_checkpoint_file(output_folder)
    load_checkpoint(checkpoint_file, datamodule, model)

    # stores all information we can think of, to reproduce this later
    save_json_metadata(
        output_file=output_folder / "train.meta.json",
        datamodule=datamodule,
        model=model,
        output_folder=output_folder,
        device_manager=device_manager,
        epochs=epochs,
        batch_size=batch_size,
        accumulate_grad_batches=accumulate_grad_batches,
        drop_incomplete_batch=drop_incomplete_batch,
        validation_period=validation_period,
        cache_samples=cache_samples,
        seed=seed,
        parallel=parallel,
        monitoring_interval=monitoring_interval,
    )

    logger.info(f"Training for at most {epochs} epochs.")

    run(
        model=model,
        datamodule=datamodule,
        validation_period=validation_period,
        device_manager=device_manager,
        max_epochs=epochs,
        output_folder=output_folder,
        monitoring_interval=monitoring_interval,
        accumulate_grad_batches=accumulate_grad_batches,
        checkpoint=checkpoint_file,
    )
